﻿// Copyright (C) 2016 - CASIO Europe GmbH
//
// This file is part of FirmwareInfo.
//
// FirmwareInfo is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
///////////////////////////////////////////////////////////////////
using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Microsoft.Win32;
using System.Reflection;
using System.IO;

namespace FirmwareInfo
{
    public partial class MainForm : Form
    {
        private const string Casio1 = "Casio Europe GmbH ";
        private const string Casio2 = "CASIO ";
        private const int IT300 = 15;
        private const int ITG500WEC = 21;
        private const int ITG500WEH = 22;
        private const string FirmwareInfoFile = "\\FlashDisk\\FirmwareInfo.txt";

        private int scaling = 2;
        
        public MainForm()
        {
            InitializeComponent();

            Version version = Assembly.GetExecutingAssembly().GetName().Version;
            AddEntry("Firmware Info", String.Format("{0}.{1}{2}", version.Major, version.Minor, version.Build));
            
            int model = GetModel();
            switch (model)
            {
                case IT300:
                    Populate();
                    break;
                case ITG500WEH:
                    Populate();
                    break;
                case ITG500WEC:
                    WindowState = FormWindowState.Maximized;
                    Menu = null;
                    Populate();
                    break;
                default:
                    break;
            }

            SaveToFile();
        }

        private int GetModel()
        {
            int result = 0;

            try
            {
                int model = 0;
                int version = 0;
                int platform = 0;
                NativeMethods.SysGetModelName(ref model, ref version, ref platform);
                result = model;
            }
            catch
            {
            }

            return result;
        }

        protected override void ScaleControl(SizeF factor, BoundsSpecified specified)
        {
            if (factor.ToSize().Width == 2)
            {
                this.scaling = 2;
            }
            else
            {
                this.scaling = 1;
            }

            base.ScaleControl(factor, specified);
        }

        private void Populate()
        {
            try
            {
                using (RegistryKey rkPlatform = Registry.LocalMachine.OpenSubKey("Platform"))
                {
                    if (rkPlatform != null)
                    {
                        var name = rkPlatform.GetValue("Name");
                        if (name != null)
                        {
                            this.statusBar.Text = String.Format("{0} {1}", name, GetDeviceIdCode());
                        }
                        var buildID = rkPlatform.GetValue("Build");
                        if (buildID != null)
                        {
                            AddEntry("OS Build ID", buildID.ToString());
                        }
                    }
                }
                using (RegistryKey rkCasio = Registry.LocalMachine.OpenSubKey("Software\\Casio\\Service Pack"))
                {
                    if (rkCasio != null)
                    {
                        var servicePackVersion = rkCasio.GetValue("Version") as string;
                        AddEntry("Service Pack", servicePackVersion);
                    }
                }
                using (RegistryKey rkApps = Registry.LocalMachine.OpenSubKey("Software\\Apps"))
                {
                    if (rkApps != null)
                    {
                        int ndx = 0;
                        string module = "";
                        string version = "";
                        string[] subKeys = rkApps.GetSubKeyNames();
                        foreach (string s in subKeys)
                        {
                            if (s.StartsWith(Casio1, StringComparison.InvariantCultureIgnoreCase) == true)
                            {
                                ndx = Casio1.Length;
                            }
                            else if (s.StartsWith(Casio2, StringComparison.InvariantCultureIgnoreCase) == true)
                            {
                                ndx = Casio2.Length;
                            }
                            else
                            {
                                ndx = 0;
                            }

                            if (ndx != 0)
                            {
                                string temp = s.Substring(ndx);
                                if (temp.Length > 0)
                                {
                                    string[] tokens = temp.Split(' ');
                                    if (tokens.Length > 1)
                                    {
                                        string versionNumber = tokens[tokens.Length - 1];
                                        if (IsVersionNumber(versionNumber) == true)
                                        {
                                            version = tokens[tokens.Length - 1];
                                            module = temp.Substring(0, temp.Length - version.Length - 1);
                                        }
                                        else
                                        {
                                            module = temp;
                                            version = "";
                                        }
                                    }
                                    else
                                    {
                                        module = temp;
                                        version = "";
                                    }

                                    AddEntry(module, version);
                                }
                            }
                        }
                    }
                }
            }
            catch
            {
            }
        }

        private bool IsVersionNumber(string theString)
        {
            bool success = false;

            try
            {
                double d = Double.Parse(theString);
                success = true;
            }
            catch
            {
            }

            return success;
        }

        private void AddEntry(string theModule, string theVersion)
        {
            try
            {
                ListViewItem lvi = new ListViewItem(new string[] { theModule, theVersion });
                int ndx = this.lsvResults.Items.Add(lvi).Index;

                this.columnModule.Width = -1;
                if (this.scaling == 2)
                {
                    this.columnModule.Width += 12;
                }

                int old = this.columnVersion.Width;
                this.columnVersion.Width = -1;
                if (this.scaling == 2)
                {
                    this.columnVersion.Width += 12;
                }
                if (this.columnVersion.Width < old)
                {
                    this.columnVersion.Width = old;
                }
            }
            catch
            {
            }
        }

        private string GetDeviceIdCode()
        {
            string deviceIdCode = "";

            try
            {
                char []buffer = new char[64];

                if (1 == NativeMethods.SysGetDeviceIDCode(buffer))
                {
                    deviceIdCode = new string(buffer).TrimEnd('\0');
                }
            }
            catch
            {
            }

            return deviceIdCode;
        }

        private void SaveToFile()
        {
            try
            {
                if (this.lsvResults.Items.Count > 0)
                {
                    using (StreamWriter sw = new StreamWriter(FirmwareInfoFile))
                    {
                        if (this.statusBar.Text.Length > 0)
                        {
                            sw.WriteLine(this.statusBar.Text);
                        }

                        foreach (ListViewItem lvi in this.lsvResults.Items)
                        {
                            sw.WriteLine(String.Format("{0}: {1}", lvi.SubItems[0].Text, lvi.SubItems[1].Text));
                        }
                    }
                }
            }
            catch
            {
            }
        }
    }
}